--[[
VariableTipAngle

Specialization for different tip angles on an trailer

Author:		Ifko[nator]
Datum:		10.11.2022

Version:	v1.0

History:	v1.0 @10.11.2022 - initial implementation in FS 22

TO DO: Script optimation, make unloading speed dependent from the current tip angle and global functionality
]]

VariableTipAngle = {};

VariableTipAngle.currentModName = g_currentModName;

VariableTipAngle.STEP_25_PERCENT = 1;
VariableTipAngle.STEP_50_PERCENT = 2;
VariableTipAngle.STEP_75_PERCENT = 3;
VariableTipAngle.STEP_100_PERCENT = 4;

function getSpecByName(self, specName, currentModName)
	local spec = self["spec_" .. Utils.getNoNil(currentModName, VariableTipAngle.currentModName) .. "." .. specName];

	if spec ~= nil then
		return spec;
	end;

	return self["spec_" .. specName];
end;

function VariableTipAngle.initSpecialization()
	Vehicle.xmlSchemaSavegame:register(XMLValueType.INT, "vehicles.vehicle(?)." .. VariableTipAngle.currentModName .. ".variableTipAngle#currentStep", "Current selected tip angle.", 4);
end;

function VariableTipAngle.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function VariableTipAngle.registerFunctions(vehicleType)
	local newFunctions = {
		"setTipAngle",
		"getCurrentMaxTipAngleAndCurrentText"
	};
	
	for _, newFunction in ipairs(newFunctions) do
		SpecializationUtil.registerFunction(vehicleType, newFunction, VariableTipAngle[newFunction]);
	end;
end;

function VariableTipAngle.registerEventListeners(vehicleType)
	local functionNames = {
		"onLoad",
		"onUpdate",
		"onRegisterActionEvents"
	};

	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, VariableTipAngle);
	end;
end;

function VariableTipAngle:onLoad(savegame)
	local specVariableTipAngle = getSpecByName(self, "variableTipAngle");
	
	specVariableTipAngle.currentStep = VariableTipAngle.STEP_100_PERCENT;
	specVariableTipAngle.currentStepSync = VariableTipAngle.STEP_100_PERCENT;

	if savegame ~= nil then
		specVariableTipAngle.currentStep = savegame.xmlFile:getValue(savegame.key .. "." .. VariableTipAngle.currentModName .. ".variableTipAngle#currentStep", specVariableTipAngle.currentStep);
	end;

	self:setTipAngle(specVariableTipAngle.currentStep, false);
		
	specVariableTipAngle.l10nTexts = {};

	local l10nTexts = {
		"input_CHANGE_TIP_ANGLE_BUTTON",
		"STEP_25_PERCENT",
		"STEP_50_PERCENT",
		"STEP_75_PERCENT",
		"STEP_100_PERCENT"
	};

	for _, l10nText in pairs(l10nTexts) do
		specVariableTipAngle.l10nTexts[l10nText] = g_i18n:getText(l10nText, VariableTipAngle.currentModName);
	end;
end;

function VariableTipAngle:onRegisterActionEvents(isActiveForInput)
	if self.isClient then
		local specVariableTipAngle = getSpecByName(self, "variableTipAngle");
		
		self:clearActionEventsTable(specVariableTipAngle.actionEvents);

		if self:getIsActiveForInput(true) then
            local newFunctions = {
				"CHANGE_TIP_ANGLE_BUTTON"
			};
			
			for _, newFunction in ipairs(newFunctions) do
				local _, actionEventId = self:addActionEvent(specVariableTipAngle.actionEvents, InputAction[newFunction], self, VariableTipAngle.actionEventChangeTipAngle, false, true, false, true, nil);
			
				g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_HIGH);
				g_inputBinding:setActionEventTextVisibility(actionEventId, true);
				g_inputBinding:setActionEventActive(actionEventId, true);

				local _, currentText = self:getCurrentMaxTipAngleAndCurrentText();

				g_inputBinding:setActionEventText(actionEventId, specVariableTipAngle.l10nTexts.input_CHANGE_TIP_ANGLE_BUTTON .. " " .. currentText);
			end;
		end;
	end;
end;

function VariableTipAngle:actionEventChangeTipAngle(actionName, inputValue, callbackState, isAnalog)
	local specVariableTipAngle = getSpecByName(self, "variableTipAngle");

	if specVariableTipAngle.currentStep < VariableTipAngle.STEP_100_PERCENT then
		specVariableTipAngle.currentStepSync = specVariableTipAngle.currentStepSync + 1;
	else
		specVariableTipAngle.currentStepSync = VariableTipAngle.STEP_25_PERCENT;
	end;

	self:setTipAngle(specVariableTipAngle.currentStepSync, false);
end;

function VariableTipAngle:onUpdate(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	local specVariableTipAngle = getSpecByName(self, "variableTipAngle");
	local specTrailer = getSpecByName(self, "trailer");
	local specDischargeable = getSpecByName(self, "dischargeable");
	local specAnimatedVehicle = getSpecByName(self, "animatedVehicle");

	local tipSide = specTrailer.tipSides[specTrailer.preferedTipSideIndex];
	
	local actionEventChangeTipAngleButton = specVariableTipAngle.actionEvents[InputAction.CHANGE_TIP_ANGLE_BUTTON];
	
	if tipSide ~= nil and actionEventChangeTipAngleButton ~= nil then
		local animation = specAnimatedVehicle.animations[tipSide.animation.name];
		local dischargeNode = specDischargeable.currentDischargeNode;
		
		local currentMaxTipAngle, currentText = self:getCurrentMaxTipAngleAndCurrentText();
		
		if animation ~= nil then
			if specTrailer.tipState == Trailer.TIPSTATE_CLOSED or specTrailer.tipState == Trailer.TIPSTATE_OPENING then
				animation.stopTime = animation.duration * currentMaxTipAngle;
			elseif specTrailer.tipState == Trailer.TIPSTATE_CLOSING then
				animation.stopTime = nil;
			end;
		end;

		if dischargeNode ~= nil then
			if dischargeNode.savedEmptySpeed == nil then
				dischargeNode.savedEmptySpeed = dischargeNode.emptySpeed;
			end;
			
			dischargeNode.emptySpeed = dischargeNode.savedEmptySpeed * currentMaxTipAngle;
		end;
		
		g_inputBinding:setActionEventText(actionEventChangeTipAngleButton.actionEventId, specVariableTipAngle.l10nTexts.input_CHANGE_TIP_ANGLE_BUTTON .. " " .. currentText);
		g_inputBinding:setActionEventActive(actionEventChangeTipAngleButton.actionEventId, specTrailer.tipState == Trailer.TIPSTATE_CLOSED);
	end;
end;

function VariableTipAngle:getCurrentMaxTipAngleAndCurrentText()
	local specVariableTipAngle = getSpecByName(self, "variableTipAngle");

	local currentMaxTipAngle = 0.25;
	local currentText = specVariableTipAngle.l10nTexts.STEP_25_PERCENT;

	if specVariableTipAngle.currentStep == VariableTipAngle.STEP_50_PERCENT then
		currentText = specVariableTipAngle.l10nTexts.STEP_50_PERCENT;
		currentMaxTipAngle = 0.5;
	elseif specVariableTipAngle.currentStep == VariableTipAngle.STEP_75_PERCENT then
		currentText = specVariableTipAngle.l10nTexts.STEP_75_PERCENT;
		currentMaxTipAngle = 0.75;
	elseif specVariableTipAngle.currentStep == VariableTipAngle.STEP_100_PERCENT then
		currentText = specVariableTipAngle.l10nTexts.STEP_100_PERCENT;
		currentMaxTipAngle = 1;
	end;

	return currentMaxTipAngle, currentText;
end;

function VariableTipAngle:setTipAngle(currentStep, noEventSend)
	local specVariableTipAngle = getSpecByName(self, "variableTipAngle");
	
	if currentStep ~= specVariableTipAngle.currentStep then
		specVariableTipAngle.currentStep = currentStep;
		
		VariableTipAngleSetTipAngleEvent.sendEvent(self, currentStep, noEventSend);
	end;
end;

function VariableTipAngle:saveToXMLFile(xmlFile, key)
	local specVariableTipAngle = getSpecByName(self, "variableTipAngle");
	
	xmlFile:setValue(key .. "#currentStep", specVariableTipAngle.currentStep);
end;

--------------------------------------------------------------------------------------------------------------------------
-----------------------------------------VariableTipAngleSetTipAngleEvent-------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------

VariableTipAngleSetTipAngleEvent = {};

local VariableTipAngleSetTipAngleEvent_mt = Class(VariableTipAngleSetTipAngleEvent, Event);

InitEventClass(VariableTipAngleSetTipAngleEvent, "VariableTipAngleSetTipAngleEvent");

function VariableTipAngleSetTipAngleEvent.emptyNew()
	local self = Event.new(VVariableTipAngleSetTipAngleEvent_mt);
    
	return self;
end;

function VariableTipAngleSetTipAngleEvent.new(trailer, currentStep)
	local self = VariableTipAngleSetTipAngleEvent.emptyNew();
	
	self.trailer = trailer;
	self.currentStep = currentStep;

	return self;
end;

function VariableTipAngleSetTipAngleEvent:readStream(streamId, connection)
	self.trailer = NetworkUtil.readNodeObject(streamId);
	self.currentStep = streamReadUInt8(streamId);
	
	self:run(connection);
end;

function VariableTipAngleSetTipAngleEvent:writeStream(streamId, connection)
	NetworkUtil.writeNodeObject(streamId, self.trailer);

	streamWriteUInt8(streamId, self.currentStep);
end;

function VariableTipAngleSetTipAngleEvent:run(connection)
	if not connection:getIsServer() then
		g_server:broadcastEvent(VariableTipAngleSetTipAngleEvent.new(self.trailer, self.currentStep), nil, connection, self.trailer);
	end;
	
    if self.trailer ~= nil then
        self.trailer:setTipAngle(self.currentStep, true);
	end;
end;

function VariableTipAngleSetTipAngleEvent.sendEvent(trailer, currentStep, noEventSend)
	if not noEventSend then
		if g_server ~= nil then
			g_server:broadcastEvent(VariableTipAngleSetTipAngleEvent.new(trailer, currentStep), nil, nil, trailer);
		else
			g_client:getServerConnection():sendEvent(VariableTipAngleSetTipAngleEvent.new(trailer, currentStep));
		end;
	end;
end;